"""
This module contains tests for the backends of the edxapp_wrapper.
"""


# pylint: disable=import-outside-toplevel,unused-import
def test_current_settings_code_imports():
    """
    Running this imports means that our backends import the right signature
    """
    import eox_tagging.edxapp_wrappers.backends.bearer_authentication_i_v1
    import eox_tagging.edxapp_wrappers.backends.course_overview_i_v1
    import eox_tagging.edxapp_wrappers.backends.enrollment_l_v1



""" Test classes for API Permissions. """
from django.contrib.auth.models import Permission, User
from django.test import TestCase, override_settings
from mock import Mock
from rest_framework import exceptions

from eox_tagging.api.v1.permissions import EoxTaggingAPIPermission, load_permissions


@override_settings(EOX_TAGGING_LOAD_PERMISSIONS=True)
class TestAPIPermissions(TestCase):
    """Test class for tags viewset."""

    def setUp(self):
        """ Permissions setup."""
        load_permissions()

        # User with permission to acccess the API
        user_permission = Permission.objects.get(codename="can_call_eox_tagging")
        self.user_authorized = User.objects.create(username="user_with_permission")
        self.user_authorized.user_permissions.add(user_permission)

        # Common user without permission
        self.common_user = User.objects.create(username="user_without_permission")

        self.has_permission = EoxTaggingAPIPermission().has_permission

        self.mock_request = Mock()
        self.view = Mock()

    def test_api_access_success(self):
        """Used to test that an authorized user can access to the Tag API."""
        self.mock_request.get_host.return_value = "test.com"
        self.mock_request.user = self.user_authorized
        self.mock_request.auth.client.url = "test.com"

        has_permission = self.has_permission(self.mock_request, self.view)

        self.assertTrue(has_permission)

    def test_api_access_denied(self):
        """Used to test that a unauthorized user can't access to the Tag API."""
        self.mock_request.get_host.return_value = "test.com"
        self.mock_request.user = self.common_user
        self.mock_request.auth.client.url = "test.com"

        has_permission = self.has_permission(self.mock_request, self.view)

        self.assertFalse(has_permission)

    def test_api_access_with_bad_host(self):
        """Used to test that an authorized user without a matching host with client can't access the API."""
        self.mock_request.get_host.return_value = "test_.com"
        self.mock_request.user = self.user_authorized
        self.mock_request.auth.client.url = "test.com"
        self.mock_request.auth.application.redirect_uri_allowed.return_value = False

        with self.assertRaises(exceptions.NotAuthenticated):
            self.has_permission(self.mock_request, self.view)

    def test_api_access_without_valid_host(self):
        """Used to test that an authorized user without a valid host can't access the API."""
        self.mock_request.get_host.return_value = None
        self.mock_request.user = self.user_authorized
        self.mock_request.auth.client.url = "test.com"
        self.mock_request.auth.application.redirect_uri_allowed.return_value = False

        with self.assertRaises(exceptions.NotAuthenticated):
            self.has_permission(self.mock_request, self.view)

    def test_api_access_without_valid_client(self):
        """Used to test that an authorized user without a valid client can't access the API."""
        self.mock_request.get_host.return_value = "test_.com"
        self.mock_request.user = self.user_authorized
        self.mock_request.auth.client.url = None
        self.mock_request.auth.application.redirect_uri_allowed.return_value = False

        with self.assertRaises(exceptions.NotAuthenticated):
            self.has_permission(self.mock_request, self.view)



"""
Test classes for Tags model
"""
import datetime

from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.test import TestCase, override_settings
from django.utils import timezone
from mock import Mock, patch
from opaque_keys.edx.keys import CourseKey

from eox_tagging.constants import AccessLevel
from eox_tagging.models import OpaqueKeyProxyModel, Tag, TagQuerySet


@override_settings(
    EOX_TAGGING_DEFINITIONS=[
        {
            "tag_type": "example_tag_1",
            "validate_owner_object": "User",  # default = Site
            "validate_target_object": "User",
            "validate_access": {"equals": "PRIVATE"},
            "validate_tag_value": {"in": ["example_tag_value", "example_tag_value_1"]},
        },
        {
            "tag_type": "example_tag_2",
            "owner_object": "Site",
            "validate_target_object": "User",
            "validate_tag_value": {"opaque_key": "CourseKey"},
        },
        {
            "tag_type": "example_tag_3",
            "validate_tag_value": {"regex": r".*eduNEXT$"},
            "validate_target_object": "User",
            "validate_expiration_date": {"exist": True},
        },
        {
            "tag_type": "example_tag_4",
            "validate_tag_value": {"in": ["example_tag_value", "example_tag_value_2"]},
            "validate_target_object": "User",
        }
    ])
class TestTag(TestCase):
    """Class for testing the Tag model."""

    def setUp(self):
        """ Model setup used to create objects used in tests."""
        self.target_object = User.objects.create(username="Tag")
        self.owner_object = User.objects.create(username="User")
        self.fake_owner_object = Site.objects.create()
        self.course_key = CourseKey.from_string('course-v1:edX+FUN101x+3T2017')

        self.test_tag = Tag.objects.create_tag(
            tag_value="example_tag_value",
            tag_type="example_tag_1",
            target_object=self.target_object,
            owner_object=self.owner_object,
            access=AccessLevel.PRIVATE,
        )

    @override_settings(
        EOX_TAGGING_DEFINITIONS=[
            {
                "tag_type": "example_tag_4",
                "validate_tag_value": {"belongs": ["example_tag_value", "example_tag_value_2"]},
                "validate_target_object": "OpaqueKeyProxyModel",
            }
        ])
    def test_bad_validation_config(self):
        """
        Used to check that if the validation is not defined then the creation fails.
        In this case, `belongs` is not defined.
        """
        with self.assertRaises(ValidationError):
            Tag.objects.create_tag(
                tag_value="example_tag_value",
                tag_type="example_tag_4",
                target_object=self.target_object,
                owner_object=self.fake_owner_object
            )

    @override_settings(
        EOX_TAGGING_DEFINITIONS=[
            {
                "tag_type": "example_tag_4",
                "validate_tag_name": {"in": ["example_tag_value", "example_tag_value_2"]},
                "validate_target_object": "OpaqueKeyProxyModel",
            }
        ])
    def test_bad_field_config(self):
        """
        Used to check that if the validation is not defined then the creation fails.
        In this case, `tag_name` is not defined.
        """
        with self.assertRaises(ValidationError):
            Tag.objects.create_tag(
                tag_value="example_tag_value",
                tag_type="example_tag_4",
                target_object=self.target_object,
                owner_object=self.fake_owner_object
            )

    @override_settings(EOX_TAGGING_DEFINITIONS=[])
    def test_empty_setting(self):
        """
        Used to test saving without validations defined.
        If the definitions array is empty then the tag cannot be created.
        """
        with self.assertRaises(ValidationError):
            Tag.objects.create_tag(
                tag_value="example_tag_value",
                tag_type="example_tag_1",
                target_object=self.target_object,
                owner_object=self.owner_object,
                access=AccessLevel.PRIVATE,
            )

    @override_settings(
        EOX_TAGGING_DEFINITIONS=[
            {
                "tag_type": "subscription_tier",
                "force_access": "private",
                "validate_tag_value": {"in": ["free", "private"]},
                "validate_owner_object": "User",
                "validate_target_object": "User",
            },
        ])
    def test_force_access_configuration(self):
        """
        Used to test that if a field is `forced` then it must be set to the value
        specified."""
        tag = Tag.objects.create_tag(
            tag_value="free",
            tag_type="subscription_tier",
            target_object=self.target_object,
            owner_object=self.owner_object,
        )

        self.assertIsNotNone(tag.id)
        self.assertEqual(tag.access.name, "PRIVATE")

    @override_settings(
        EOX_TAGGING_DEFINITIONS=[
            {
                "tag_type": "subscription_tier",
                "force_activation_date": "2020-10-19 10:20:30",
                "validate_tag_value": {"in": ["free", "private"]},
                "validate_owner_object": "User",
                "validate_target_object": "User",
            },
        ])
    def test_force_activation_date_configuration(self):
        """Used to test that if a field is `forced` the must be set to the value specified."""
        tag = Tag.objects.create_tag(
            tag_value="free",
            tag_type="subscription_tier",
            target_object=self.target_object,
            owner_object=self.owner_object,
        )

        self.assertIsNotNone(tag.id)
        self.assertEqual(str(tag.activation_date), "2020-10-19 10:20:30")

    @override_settings(
        EOX_TAGGING_DEFINITIONS=[
            {
                "tag_type": "subscription_tier",
                "force_expiration_date": "2020-10-19 10:20:30",
                "validate_tag_value": {"in": ["free", "private"]},
                "validate_owner_object": "User",
                "validate_target_object": "User",
            },
        ])
    def test_force_expiration_date_configuration(self):
        """Used to test that if a field is `forced` the must be set to the value specified."""
        tag = Tag.objects.create_tag(
            tag_value="free",
            tag_type="subscription_tier",
            target_object=self.target_object,
            owner_object=self.owner_object,
        )

        self.assertIsNotNone(tag.id)
        self.assertEqual(str(tag.expiration_date), "2020-10-19 10:20:30")

    @override_settings(
        EOX_TAGGING_DEFINITIONS=[
            {
                "tag_type": "subscription_tier",
                "force_tag_value": "free",
                "validate_owner_object": "User",
                "validate_target_object": "User",
            },
        ])
    def test_force_tag_value_configuration(self):
        """Used to test that if a field is `forced` the must be set to the value specified."""
        tag = Tag.objects.create_tag(
            tag_type="subscription_tier",
            target_object=self.target_object,
            owner_object=self.owner_object,
        )

        self.assertIsNotNone(tag.id)
        self.assertEqual(tag.tag_value, "free")

    @override_settings(
        EOX_TAGGING_DEFINITIONS=[
            {
                "tag_type": "example_tag_7",
                "validate_owner_object": "User",
                "validate_access": {"equals": "PRIVATE"},
                "validate_tag_value": {"in": ["example_tag_value", "example_tag_value_1"]},
            }]
    )
    def test_create_config_without_target(self):
        """
        Used to test creating a Tag without target.
        It results in validation error.
        """
        with self.assertRaises(ValidationError):
            Tag.objects.create_tag(
                tag_value="example_tag_value",
                tag_type="example_tag_7",
                target_object=self.target_object,
                owner_object=self.owner_object,
                access=AccessLevel.PRIVATE,
            )

    def test_valid_tag(self):
        """ Used to confirm that the tags created are valid."""
        tag_status = getattr(self.test_tag, "status")

        self.assertEqual(tag_status, 1)

    def test_tag_value(self):
        """ Used to confirm that the tag_value is correct."""
        tag_value = getattr(self.test_tag, "tag_value")

        self.assertEqual(tag_value, "example_tag_value")

    def test_tag_type(self):
        """ Used to confirm that the tag_type is correct."""
        tag_value = getattr(self.test_tag, "tag_type")

        self.assertEqual(tag_value, "example_tag_1")

    def test_tag_value_not_in_settings(self):
        """
        Used to confirm validation error when the value is not defined in settings.
        If the key validate_tag_value or tag_value is defined in the config settings
        of the tag, then tag_value must exist or match the validation defined.
        """
        with self.assertRaises(ValidationError):
            Tag.objects.create_tag(
                tag_value="testValues",
                tag_type="example_tag_1",
                target_object=self.target_object,
                owner_object=self.owner_object,
            )

    def test_tag_type_not_in_settings(self):
        """
        Used to confirm validation error when the value is not defined in settings.
        Due to tag_type is a required field, it must exist in any case and match the
        value defined.
        """
        with self.assertRaises(ValidationError):
            Tag.objects.create_tag(
                tag_value="example_tag_value",
                tag_type="testTypes",
                target_object=self.target_object,
                owner_object=self.owner_object,
            )

    def test_tag_different_generic_objects_fail(self):
        """
        Used to confirm that tags can't be created if the target_object does not match
        the definition.
        """
        with self.assertRaises(ValidationError):
            Tag.objects.create_tag(
                tag_value="course-v1:demo-courses+DM101+2017",
                tag_type="example_tag_2",
                target_object=self.fake_owner_object,  # Is site not user as defined
                owner_object=self.fake_owner_object,
            )

    def test_tag_validation_regex_accepts_pattern(self):
        """ Used to confirm that tags can accept a pattern if defined in settings."""
        Tag.objects.create_tag(
            tag_value="example by eduNEXT",
            tag_type="example_tag_3",
            target_object=self.target_object,
            owner_object=self.fake_owner_object,
            expiration_date=datetime.date(2020, 10, 19),
        )

    def test_tag_validation_regex_accepts_pattern_fail(self):
        """ Used to confirm that saving fails if tag does not match pattern defined in settings."""
        with self.assertRaises(ValidationError):
            Tag.objects.create_tag(
                tag_value="example by edx",
                tag_type="example_tag_3",
                target_object=self.target_object,
                owner_object=self.owner_object,
                expiration_date=datetime.date(2020, 10, 19),
            )

    def test_tag_validation_owner_must_be_site(self):
        """ Used to confirm that a tag must have a site as owner."""
        with self.assertRaises(ValidationError):
            Tag.objects.create_tag(
                tag_value="example by eduNEXT",
                tag_type="example_tag_3",
                target_object=self.target_object,
                owner_object=self.target_object,  # Is user and not site as specified
                expiration_date=datetime.date(2020, 10, 19),
            )

    def test_tag_immutable(self):
        """ Used to confirm that the tags can't be updated."""
        setattr(self.test_tag, "tag_value", "value")
        with self.assertRaises(ValidationError):
            self.test_tag.save()

    def test_find_by_owner(self):
        """ Used to confirm that can retrieve tags by owner_object."""
        tags_owned = Tag.objects.find_by_owner(owner_type="user", owner_id={"username": "User"})

        self.assertEqual(tags_owned.first().owner_object_id, self.owner_object.id)

    def test_find_all_tags_for(self):
        """Used to confirm that can retrieve tags by target object."""
        tags = Tag.objects.find_all_tags_for(target_type="user", target_id={"username": "Tag"})

        self.assertEqual(tags.first().target_object_id, self.target_object.id)

    def test_tag_soft_delete(self):
        """
        Used to confirm that the tags can be invalidated soft deleting them.
        This also means that they will no longer exist in active tags set.
        """
        self.test_tag.delete()

        deleted_tag = Tag.objects.inactive().get(id=self.test_tag.id)
        with self.assertRaises(ObjectDoesNotExist):
            Tag.objects.active().get(id=self.test_tag.id)
        self.assertGreater(timezone.now(), deleted_tag.inactivated_at)

    def test_create_tag_without_target_object(self):
        """
        Used to test that a tag can't be created without a target.
        """
        with self.assertRaises(ValidationError):
            Tag.objects.create_tag(
                tag_value="example_tag_value",
                tag_type="example_tag_4",
                owner_object=self.fake_owner_object,
            )

    def test_create_with_default_owner(self):
        """
        Used to test that if the configuration does not have an owner defined, then
        the tag must belong to a site.
        """
        Tag.objects.create_tag(
            tag_value="example_tag_value",
            tag_type="example_tag_4",
            target_object=self.target_object,
            owner_object=self.fake_owner_object,
        )

    def test_create_without_matching_default_owner(self):
        """
        Used to test that if the configuration does not have an owner defined, then
        the tag must belong to a site.
        """
        with self.assertRaises(ValidationError):
            Tag.objects.create_tag(
                tag_value="example_tag_value",
                tag_type="example_tag_4",
                target_object=self.target_object,
                owner_object=self.owner_object,
            )

    def test_create_without_default_owner(self):
        """
        Used to test that if the configuration does not have an owner defined, then
        the tag must belong to a site.
        """
        with self.assertRaises(ValidationError):
            Tag.objects.create_tag(
                tag_value="example_tag_value",
                tag_type="example_tag_4",
                target_object=self.target_object,
            )

    @override_settings(
        EOX_TAGGING_DEFINITIONS=[
            {
                "tag_type": "example_tag_3",
                "validate_tag_value": {"regex": r".*eduNEXT$"},
                "validate_target_object": "User",
                "validate_expiration_date": {"exist": True, "between": ["2020-10-19 10:20:30", "2020-12-04 10:20:30"]},
                "validate_activation_date": "2020-06-16 10:20:30",
            }]
    )
    def test_validation_date_between(self):
        """
        Used to test date validations using BETWEEN validator.
        This means that expiration_date must be grater or equal than "2020-10-19 10:20:30",
        or less or equal than "2020-12-04 10:20:30".
        """
        tag = Tag.objects.create_tag(
            tag_value="example by eduNEXT",
            tag_type="example_tag_3",
            target_object=self.target_object,
            owner_object=self.fake_owner_object,
            expiration_date=datetime.datetime(2020, 10, 19, 10, 20, 30),
            activation_date=datetime.datetime(2020, 6, 16, 10, 20, 30),
        )

        self.assertIsNotNone(tag.id)

    @override_settings(
        EOX_TAGGING_DEFINITIONS=[
            {
                "tag_type": "example_tag_3",
                "validate_tag_value": {"regex": r".*eduNEXT$"},
                "validate_target_object": "User",
                "validate_expiration_date": {"exist": True, "between": ["2020-10-19 10:20:30", "2020-12-04 10:20:30"]},
                "validate_activation_date": "2020-06-16 10:20:30",
            }]
    )
    def test_validation_date_not_in_between(self):
        """
        Used to test date validations using BETWEEN validator. In this case, the validator must
        raise a validation error because the date is not in between the two dates defined.
        """
        with self.assertRaises(ValidationError):
            Tag.objects.create_tag(
                tag_value="example by eduNEXT",
                tag_type="example_tag_3",
                target_object=self.target_object,
                owner_object=self.fake_owner_object,
                expiration_date=datetime.datetime(2020, 5, 19, 10, 20, 30),
                activation_date=datetime.datetime(2020, 6, 16, 10, 20, 30),
            )

    @override_settings(
        EOX_TAGGING_DEFINITIONS=[
            {
                "tag_type": "example_tag_3",
                "validate_tag_value": {"regex": r".*eduNEXT$"},
                "validate_target_object": "User",
                "validate_expiration_date": {"in": ["2020-12-04 10:20:30", "2020-10-19 10:20:30"]},
                "validate_activation_date": "2020-06-16 10:20:30",
            }]
    )
    def test_validation_date_in_list(self):
        """
        Used to test date validations using IN validator. This means that expiration_date bust be equal to
        "2020-12-04 10:20:30" or equal to "2020-10-19 10:20:30".
        """
        tag = Tag.objects.create_tag(
            tag_value="example by eduNEXT",
            tag_type="example_tag_3",
            target_object=self.target_object,
            owner_object=self.fake_owner_object,
            expiration_date=datetime.datetime(2020, 10, 19, 10, 20, 30),
            activation_date=datetime.datetime(2020, 6, 16, 10, 20, 30),
        )

        self.assertIsNotNone(tag.id)

    @override_settings(
        EOX_TAGGING_DEFINITIONS=[
            {
                "tag_type": "example_tag_3",
                "validate_tag_value": {"regex": r".*eduNEXT$"},
                "validate_target_object": "User",
                "validate_expiration_date": {"in": ["2020-12-04 10:20:30", "2020-10-19 10:20:30"]},
                "validate_activation_date": "2020-06-16 10:20:30",
            }]
    )
    def test_validation_date_not_in_list(self):
        """
        Used to test date validations using IN validator. In this case, the validator must raise
        a validation error because the date is different two those defined inside the array.
        """
        with self.assertRaises(ValidationError):
            Tag.objects.create_tag(
                tag_value="example by eduNEXT",
                tag_type="example_tag_3",
                target_object=self.target_object,
                owner_object=self.fake_owner_object,
                expiration_date=datetime.datetime(2020, 4, 19, 10, 20, 30),
                activation_date=datetime.datetime(2020, 6, 16, 10, 20, 30),
            )

    def test_soft_delete_queryset(self):
        """Used to test deactivating tags using a tag queryset."""
        Tag.objects.delete()

        self.assertFalse(Tag.objects.active())


class TestTagQuerysetManager(TestCase):
    """
    Test cases for queryset used as tag manager. This test cases are focused to
    check how the method behaves with objects like CourseOverview and CourseEnrollment.
    """

    def setUp(self):
        self.tag_query_set = TagQuerySet()
        self.course_id = "course-v1:edX+DemoX+Demo_Course"

    @patch.object(OpaqueKeyProxyModel, 'objects')
    def test_create_tag_with_course(self, opaque_objects_mock):
        """Test tagging a course using TagQueryset manager."""
        self.tag_query_set.create = Mock()
        course_mock = Mock()
        opaque_mock = Mock()
        opaque_objects_mock.get_or_create.return_value = opaque_mock, Mock()
        course_mock.__class__.__name__ = "CourseOverview"
        kwargs = {
            "target_object": course_mock,
        }

        self.tag_query_set.create_tag(**kwargs)

        self.tag_query_set.create.assert_called_once_with(target_object=opaque_mock)

    @patch.object(TagQuerySet, '_get_object_for_this_type')
    def test_find_all_tags_for_course(self, _get_object_for_this_type):
        """Test getting tags with course as target."""
        target_type, target_id = "CourseOverview", 1
        target, target_ctype = Mock(), Mock()
        _get_object_for_this_type.return_value = target, target_ctype
        self.tag_query_set.filter = Mock()
        target.values_list.return_value = [1]

        self.tag_query_set.find_all_tags_for(target_type, target_id)

        _get_object_for_this_type.assert_called_once_with(
            "opaquekeyproxymodel",
            target_id
        )
        self.tag_query_set.filter.assert_called_once_with(
            target_type=target_ctype,
            target_object_id__in=[1],
        )

    @patch.object(ContentType, 'objects')
    def test_get_objects_for_type_user(self, content_type_mock):
        """Test getting courses associated with tags."""
        ctype_object = Mock()
        content_type_mock.get.return_value = ctype_object
        object_type, object_id = "User", {
            "username": "username",
        }

        self.tag_query_set._get_object_for_this_type(  # pylint: disable=protected-access
            object_type,
            object_id,
        )

        content_type_mock.get.assert_called_once_with(model=object_type)
        ctype_object.get_all_objects_for_this_type.assert_called_once_with(
            username="username",
        )

    @patch.object(ContentType, 'objects')
    def test_get_objects_for_type_course(self, content_type_mock):
        """Test getting courses associated with tags."""
        ctype_object = Mock()
        content_type_mock.get.return_value = ctype_object
        object_type, object_id = "OpaqueKeyProxyModel", {
            "course_id": self.course_id,
        }
        object_id_modified = {
            "opaque_key": CourseKey.from_string(self.course_id),
        }

        self.tag_query_set._get_object_for_this_type(  # pylint: disable=protected-access
            object_type,
            object_id,
        )

        content_type_mock.get.assert_called_once_with(model=object_type)
        ctype_object.get_all_objects_for_this_type.assert_called_once_with(
            **object_id_modified
        )

    @patch.object(ContentType, 'objects')
    def test_get_objects_for_type_enrollment(self, content_type_mock):
        """Test getting enrollments associated with tags."""
        ctype_object = Mock()
        content_type_mock.get.return_value = ctype_object
        object_type, object_id = "CourseEnrollment", {
            "course_id": self.course_id,
            "username": "username",
        }
        object_id_modified = {
            "course_id": CourseKey.from_string(self.course_id),
            "user__username": "username",
        }

        self.tag_query_set._get_object_for_this_type(   # pylint: disable=protected-access
            object_type,
            object_id,
        )

        content_type_mock.get.assert_called_once_with(model=object_type)
        ctype_object.get_all_objects_for_this_type.assert_called_once_with(
            **object_id_modified
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2020-07-07 15:49
from __future__ import unicode_literals

from django.db import migrations, models

import eox_tagging.constants


class Migration(migrations.Migration):

    dependencies = [
        ('eox_tagging', '0002_auto_20200630_1333'),
    ]

    operations = [
        migrations.RenameField(
            model_name='tag',
            old_name='invalidated_at',
            new_name='inactivated_at',
        ),
        migrations.AlterField(
            model_name='tag',
            name='status',
            field=models.PositiveIntegerField(choices=[(1, 'ACTIVE'), (0, 'INACTIVE')], default=eox_tagging.constants.Status(1), editable=False),
        ),
    ]



"""
Backend CourseOverview file, here are all the methods from
openedx.core.djangoapps.content.course_overviews.
"""

from unittest.mock import Mock


def get_course_overview():
    """Backend to get course overview."""

    return Mock()



"""This file contains all the test for the general views.py file.

Classes:
    EOXInfoTestCase: Test info_view.
"""
from os.path import dirname, realpath
from subprocess import CalledProcessError, check_output

from django.test import TestCase
from django.urls import reverse
from mock import patch
from rest_framework import status

import eox_tagging
from eox_tagging import views


class EOXInfoTestCase(TestCase):
    """Possible tests scenarios for info_view."""

    def setUp(self):
        """Setup common conditions for every test case"""
        self.url = reverse('eox-info')
        self.view_directory = dirname(realpath(views.__file__))

    def test_view_info_accessible(self):
        """
        This method tests the desired behavior of info_view when this
        does not raise any exception.

        Expected behavior:
            - Return expected content.
            - Status code 200.
        """
        git_data = check_output(
            ['git', 'rev-parse', 'HEAD'],
            cwd=self.view_directory,
        )
        expected_result = {
            'version': eox_tagging.__version__,
            'name': 'eox-tagging',
            'git': git_data.decode().rstrip('\r\n'),
        }

        response = self.client.get(self.url)

        content = response.json()
        self.assertEqual(expected_result, content)
        self.assertEqual(status.HTTP_200_OK, response.status_code)

    @patch('eox_tagging.views.check_output')
    def test_view_info_response_data(self, check_output_mock):
        """
        This method tests the desired behavior of info_view when
        raise a CalledProcessError exception.

        Expected behavior:
            - check_output called once with the right values.
            - Return expected content.
            - Status code 200.
        """
        check_output_mock.side_effect = CalledProcessError(
            cmd='test-error',
            returncode=0,
        )
        expected_result = {
            'version': eox_tagging.__version__,
            'name': 'eox-tagging',
            'git': '',
        }

        response = self.client.get(self.url)

        content = response.json()
        check_output_mock.assert_called_once_with(
            ['git', 'rev-parse', 'HEAD'],
            cwd=self.view_directory,
        )
        self.assertEqual(expected_result, content)
        self.assertEqual(status.HTTP_200_OK, response.status_code)



"""
Integration test for EOX Info view.
"""
import requests
from django.conf import settings as ds
from django.test import TestCase
from django.urls import reverse
from rest_framework import status

settings = ds.INTEGRATION_TEST_SETTINGS


class TestInfoView(TestCase):
    """
    Integration test suite for the info view.
    """

    def test_info_view_success(self) -> None:
        """Test the info view.

        Expected result:
        - The status code is 200.
        - The response contains the version, name and git commit hash.
        """
        url = f"{settings['EOX_TAGGING_BASE_URL']}{reverse('eox-info')}"

        response = requests.get(url, timeout=settings["API_TIMEOUT"])

        response_data = response.json()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn("version", response_data)
        self.assertIn("name", response_data)
        self.assertIn("git", response_data)



"""
Common Django settings for eox_tagging project.

For more information on this file, see
https://docs.djangoproject.com/en/1.11/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.11/ref/settings/
"""
from __future__ import unicode_literals

from importlib.util import find_spec

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.11/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'secret-key'


# Application definition

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'rest_framework',
    'eox_tagging',
]

EOX_AUDIT_MODEL_APP = 'eox_audit_model.apps.EoxAuditModelConfig'

ROOT_URLCONF = 'eox_tagging.urls'

# Internationalization
# https://docs.djangoproject.com/en/1.11/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_TZ = True

ALLOWED_HOSTS = ['*']

# Define AutoField to generate primary keys
# https://docs.djangoproject.com/en/3.2/releases/3.2/#customizing-type-of-auto-created-primary-keys
DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'


def plugin_settings(settings):
    """
    Set of plugin settings used by the Open Edx platform.
    More info: https://github.com/openedx/edx-platform/blob/master/openedx/core/djangoapps/plugins/README.rst
    """
    # Plugin settings
    settings.EOX_TAGGING_GET_ENROLLMENT_OBJECT = "eox_tagging.edxapp_wrappers.backends.enrollment_l_v1"
    settings.EOX_TAGGING_GET_COURSE_OVERVIEW = "eox_tagging.edxapp_wrappers.backends.course_overview_i_v1"
    settings.EOX_TAGGING_DEFINITIONS = []
    settings.EOX_TAGGING_LOAD_PERMISSIONS = True
    settings.DATA_API_DEF_PAGE_SIZE = 1000
    settings.DATA_API_MAX_PAGE_SIZE = 5000
    settings.EOX_TAGGING_BEARER_AUTHENTICATION = 'eox_tagging.edxapp_wrappers.backends.bearer_authentication_i_v1'
    if hasattr(settings, 'INSTALLED_APPS'):
        if find_spec('eox_audit_model') and EOX_AUDIT_MODEL_APP not in settings.INSTALLED_APPS:
            settings.INSTALLED_APPS.append(EOX_AUDIT_MODEL_APP)



"""
App configuration for eox_tagging.
"""
from __future__ import unicode_literals

from django.apps import AppConfig


class EoxTaggingConfig(AppConfig):
    """
    eox-tagging configuration.
    """
    name = 'eox_tagging'
    verbose_name = 'eduNEXT OpenedX Tagging'
    plugin_app = {
        'url_config': {
            'lms.djangoapp': {
                'namespace': 'eox-tagging',
                'regex': r'^eox-tagging/',
                'relative_path': 'urls',
            },
            'cms.djangoapp': {
                'namespace': 'eox-tagging',
                'regex': r'^eox-tagging/',
                'relative_path': 'urls',
            }
        },
        'settings_config': {
            'lms.djangoapp': {
                'common': {'relative_path': 'settings.common'},
                'test': {'relative_path': 'settings.test'},
                'production': {'relative_path': 'settings.production'},
            },
            'cms.djangoapp': {
                'common': {'relative_path': 'settings.common'},
                'test': {'relative_path': 'settings.test'},
                'production': {'relative_path': 'settings.production'},
            },
        }
    }



"""
Model to store tags in the database.
"""
import logging
import re
import uuid
from datetime import datetime

from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.db.models.query import QuerySet
from django.utils import timezone
from opaque_keys.edx.django.models import CourseKeyField
from opaque_keys.edx.keys import CourseKey
from six import python_2_unicode_compatible

from eox_tagging.constants import AccessLevel, Status
from eox_tagging.validators import TagValidators

log = logging.getLogger(__name__)

OPAQUE_KEY_PROXY_MODEL_TARGETS = [
    "courseoverview",
]

PROXY_MODEL_NAME = "opaquekeyproxymodel"


class TagQuerySet(QuerySet):
    """ Tag queryset used as manager."""

    def create_tag(self, **kwargs):
        """Method used to create tags."""
        target = kwargs.pop("target_object", None)
        if target and target.__class__.__name__.lower() in OPAQUE_KEY_PROXY_MODEL_TARGETS:
            kwargs["target_object"], _ = OpaqueKeyProxyModel.objects.get_or_create(opaque_key=target.id)
        else:
            kwargs["target_object"] = target
        instance = self.create(**kwargs)
        return instance

    def active(self):
        """Returns all active tags."""
        return self.filter(inactivated_at=None)

    def inactive(self):
        """Returns all inactive tags."""
        return self.exclude(inactivated_at=None)

    def delete(self):
        """Used to delete a set of tags."""
        return super().update(inactivated_at=timezone.now(), status=Status.INACTIVE)

    def find_by_owner(self, owner_type, owner_id):
        """Returns all tags owned by owner_id."""
        try:
            owner, ctype = self._get_object_for_this_type(owner_type, owner_id)
        except ObjectDoesNotExist:
            return self.none()

        owner_ids = list(owner.values_list("id", flat=True))

        return self.filter(owner_type=ctype, owner_object_id__in=owner_ids)

    def find_all_tags_for(self, target_type, target_id):
        """Returns all tags defined on an object."""
        target_type = PROXY_MODEL_NAME if target_type.lower() in OPAQUE_KEY_PROXY_MODEL_TARGETS else target_type

        try:
            target, ctype = self._get_object_for_this_type(target_type, target_id)
        except ObjectDoesNotExist:
            return self.none()

        target_ids = list(target.values_list("id", flat=True))

        return self.filter(target_type=ctype, target_object_id__in=target_ids)

    def find_all_tags_by_type(self, object_type):
        """Returns all tags with target_type equals to object_type."""
        return self.filter(target_type__model__exact=object_type)

    def hard_delete(self):
        """ Method for deleting Tag objects"""
        return super().delete()

    def _get_object_for_this_type(self, object_type, object_id):
        """
        Function that given an object type returns the correct content type and a list of objects
        associated.
        """
        ctype = ContentType.objects.get(model=object_type)

        if object_type.lower() == PROXY_MODEL_NAME:
            object_id = object_id.get("course_id")
            object_id = {
                "opaque_key": CourseKey.from_string(object_id),
            }

        if object_type.lower() in ["courseenrollment", "generatedcertificate"]:

            course_id = object_id.get("course_id")
            username = object_id.get("username")
            verify_uuid = object_id.get("verify_uuid")
            object_id = {}

            if course_id:
                object_id["course_id"] = CourseKey.from_string(course_id)

            if username:
                object_id["user__username"] = username

            if verify_uuid:
                object_id["verify_uuid"] = verify_uuid

        object_instances = ctype.get_all_objects_for_this_type(**object_id)

        return object_instances, ctype


@python_2_unicode_compatible
class OpaqueKeyProxyModel(models.Model):
    """Model used to tag objects with opaque keys."""
    opaque_key = CourseKeyField(max_length=255)
    objects = models.Manager()

    def __str__(self):
        """Method that returns the opaque_key string representation."""
        return str(self.opaque_key)


@python_2_unicode_compatible
class Tag(models.Model):
    """
    Model class for tags.

    Overrides save method to validate data entries before saving.
    Also, overrides delete so softDeletion is available.

    Attributes:
        tag_value: unicode value of the tag. Example: free or premium
        tag_type: type of the tag. Example: Subscription tiers
        access: access level of the tag
        activation_date: date to activate the tag
        expiration-date: date to deactivate de tag
        target_object: object to tag
        belongs_to: object to which the tag belongs
        status: status of the tag, valid or invalid
        invalidated_at: date when the tag is soft deleted
    """
    key = models.UUIDField(
        unique=True,
        editable=False,
        default=uuid.uuid4,
        verbose_name="Public identifier",
    )

    tag_value = models.CharField(max_length=150)
    tag_type = models.CharField(max_length=150)
    access = models.PositiveIntegerField(
        choices=AccessLevel.choices(), default=AccessLevel.PUBLIC,
    )
    activation_date = models.DateTimeField(null=True, blank=True)
    expiration_date = models.DateTimeField(null=True, blank=True)
    created_at = models.DateTimeField(auto_now_add=True, editable=False)

    status = models.PositiveIntegerField(
        choices=Status.choices(), default=Status.ACTIVE, editable=False,
    )

    inactivated_at = models.DateTimeField(blank=True, null=True, editable=False)

    # Generic foreign key for tagged objects
    target_type = models.ForeignKey(
        ContentType,
        on_delete=models.CASCADE,
        related_name="%(class)s_type",
        null=True,
        blank=True,
    )
    target_object_id = models.PositiveIntegerField(
        null=True,
        blank=True,
    )
    target_object = GenericForeignKey("target_type", "target_object_id")

    # Generic foreign key for `tag belonging to` USER or SITE
    owner_type = models.ForeignKey(
        ContentType,
        on_delete=models.CASCADE,
        related_name="owner_%(class)s_type",
        null=True,
        blank=True,
    )
    owner_object_id = models.PositiveIntegerField(
        null=True,
        blank=True,
    )
    owner_object = GenericForeignKey("owner_type", "owner_object_id")

    objects = TagQuerySet().as_manager()

    class Meta:
        """Meta class. """
        verbose_name = "tag"
        verbose_name_plural = "tags"
        app_label = "eox_tagging"

    def __str__(self):
        return str(self.tag_value)

    @property
    def target_object_type(self):
        """Obtain the name of the object target by the `Tag`."""
        return self.target_object.__class__.__name__ if self.target_object else None

    @property
    def owner_object_type(self):
        """Obtain the name of the object which the tag belongs to."""
        return self.owner_object.__class__.__name__ if self.owner_object else None

    def set_attribute(self, attr, value):
        """Function that takes a value and sets it to the instance attribute."""
        if attr == "access":
            self.access = AccessLevel.get_access_object(value)
            return

        if attr == "activation_date":
            self.activation_date = datetime.strptime(value, "%Y-%m-%d %H:%M:%S")
            return

        if attr == "expiration_date":
            self.expiration_date = datetime.strptime(value, "%Y-%m-%d %H:%M:%S")
            return

        setattr(self, attr, value)

    def get_attribute(self, attr, name=False):
        """
        Function used to format attributes getting them from self object.

        Arguments:
            - name: used if attr is target or owner and want to get the class name.
        """

        if attr and re.match(r".+object$|.+object_type$", attr):
            return self.__get_model(attr, name)

        if attr == 'access':
            return self.__get_field_choice(attr)

        return getattr(self, attr)

    def __get_model(self, attr, name):
        """
        Function that gets the type stored by the proxy model. This function is called when we want
        the actual type of the target object.

        Arguments:
            - attr: name of the attr
            - name: in case we want the class and not the object
        """

        field_value = getattr(self, f"{attr}_type")
        if name:
            return field_value
        return getattr(self, attr)

    def __get_field_choice(self, attr):
        """
        Function that gets the choice of the choice field

        Arguments:
            - attr: name of the attr
            - name: in case we want the class and not the object
        """
        field_value = getattr(self, attr)

        try:
            choice = getattr(field_value, "name")
            return choice
        except AttributeError:
            choice = AccessLevel.get_choice(field_value)
            return choice

    def clean(self):
        """
        Validates inter-field relations
        """
        self.validator.validate_fields()

    def clean_fields(self):  # pylint: disable=arguments-differ
        """
        Validates fields individually
        """
        if getattr(settings, "EOX_TAGGING_SKIP_VALIDATIONS", False):  # Skip these validations while testing
            return
        self.validator.validate_fields_integrity()

    def full_clean(self, exclude=None, validate_unique=False):  # pylint: disable=arguments-differ
        """
        Call clean_fields(), clean(), and validate_unique() -not implemented- on the model.
        Raise a ValidationError for any errors that occur.
        """
        self.validator = TagValidators(self)  # pylint: disable=attribute-defined-outside-init
        self.clean()
        self.clean_fields()

    def save(self, *args, **kwargs):
        self.full_clean()
        super().save(*args, **kwargs)

    def delete(self):  # pylint: disable=arguments-differ
        self.inactivated_at = timezone.now()
        self.status = Status.INACTIVE
        super().save()

    def hard_delete(self):
        """Deletes object from database."""
        super().delete()



"""
Serializers for tags and related objects.
"""

from django.core.exceptions import ValidationError
from django.utils.translation import gettext as _
from rest_framework import serializers

from eox_tagging.api.v1 import fields
from eox_tagging.constants import AccessLevel, Status
from eox_tagging.edxapp_accessors import get_object_from_edxapp, get_site
from eox_tagging.models import Tag

MODELS_WITH_COMPOUND_KEYS = {
    "courseenrollment": ["username", "course_id"],
    "generatedcertificate": ["username", "course_id"],
}


class TagSerializer(serializers.ModelSerializer):
    """Serializer for tag objects."""

    target_id = serializers.CharField(source="target_object", write_only=True)
    owner_id = serializers.CharField(
        source="owner_object", required=False, write_only=True
    )
    owner_type = serializers.CharField(
        source="owner_object_type", write_only=True, required=False
    )
    target_type = serializers.CharField(source="target_object_type", write_only=True)
    meta = serializers.SerializerMethodField()
    access = fields.EnumField(enum=AccessLevel, required=False)
    status = fields.EnumField(enum=Status, required=False)

    class Meta:
        """Meta class."""

        model = Tag
        fields = (
            "meta",
            "key",
            "tag_value",
            "tag_type",
            "access",
            "activation_date",
            "expiration_date",
            "target_id",
            "owner_id",
            "owner_type",
            "target_type",
            "status",
        )

    def get_meta(self, instance):
        """Getter of read-only field that returns technical information."""
        return {
            "target_id": str(instance.target_object),
            "target_type": instance.target_object_type,
            "owner_id": str(instance.owner_object),
            "owner_type": instance.owner_object_type,
            "created_at": instance.created_at,
            "inactivated_at": instance.inactivated_at,
        }

    def create(self, validated_data):
        """Function that creates a Tag instance."""
        target_object = None
        owner_object = None
        target_type = validated_data.pop("target_object_type")
        owner_type = validated_data.pop("owner_object_type", None)
        target = validated_data.pop("target_object", None)

        if target_type and target_type.lower() in MODELS_WITH_COMPOUND_KEYS:
            data = self._convert_compound_keys(target, target_type.lower())
        else:
            data = {
                "target_id": target,
            }

        try:
            target_object = get_object_from_edxapp(target_type, **data)
        except Exception as exc:
            raise serializers.ValidationError(
                {"Target": _(f"Error getting {target_type} object.")}
            ) from exc

        if owner_type and owner_type.lower() == "user":
            owner_object = self.context.get("request").user
        else:
            owner_object = get_site()

        tag_object = {
            "target_object": target_object,
            "owner_object": owner_object,
        }
        tag_object.update(validated_data)

        try:
            return Tag.objects.create_tag(**tag_object)
        except ValidationError as validation_error:
            raise serializers.ValidationError({"Tag": _(f"{validation_error.message}")})

    @staticmethod
    def _convert_compound_keys(ids, object_type):
        """
        Function that converts strings with format: `key1: key2` into a dictionary.
        """
        target_id = ids.replace(" ", "").split(":", 1)
        if len(target_id) == 1:
            return {
                "target_id": ids,
            }
        target_labels = MODELS_WITH_COMPOUND_KEYS.get(object_type)
        target_pairs = zip(target_labels, target_id)
        return dict(target_pairs)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2020-06-30 13:33
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eox_tagging', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tag',
            name='activation_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='tag',
            name='expiration_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



"""
Backend CourseEnrollments file, here is the method to access enrollments
objects.
"""


def get_enrollment_object():
    """Backend to get enrollment object."""
    from student.models import CourseEnrollment  # pylint: disable=C0415, E0401
    return CourseEnrollment



"""Filter module for tags."""
import warnings  # NOTE: to be removed alongside the backport

from django_filters import compat  # NOTE: to be removed alongside the backport
from django_filters import rest_framework as filters

from eox_tagging.constants import AccessLevel
from eox_tagging.models import Tag

FILTER_TARGET_MAPPING = {
    "course_id": {
        "object": "courseoverview",
    },
    "username": {
        "object": "user",
    },
    "certificate_verify_uuid": {
        "object": "generatedcertificate",
        "target_id": "verify_uuid",
    }
}


class TagFilter(filters.FilterSet):
    """Filter class for tags."""

    course_id = filters.CharFilter(method="filter_by_target_object")
    username = filters.CharFilter(method="filter_by_target_object")
    certificate_verify_uuid = filters.CharFilter(method="filter_by_target_object")
    target_type = filters.CharFilter(method="filter_target_types")
    created_at = filters.DateTimeFromToRangeFilter()
    activation_date = filters.DateTimeFromToRangeFilter()
    expiration_date = filters.DateTimeFromToRangeFilter()
    access = filters.CharFilter(method="filter_access_type")

    class Meta:
        """Meta class."""
        model = Tag
        fields = ["key", "status", "tag_type", "tag_value"]

    def filter_by_target_object(self, queryset, name, value):
        """Filter that returns the tags associated with target."""
        if value:

            filter_target = FILTER_TARGET_MAPPING.get(name)
            target_id_name = filter_target.get("target_id")
            name = target_id_name if target_id_name else name

            filter_params = {
                "target_type": filter_target.get("object"),
                "target_id": {
                    name: value,
                },
            }

            try:
                queryset = queryset.find_all_tags_for(
                    **filter_params
                )
            except Exception:  # pylint: disable=broad-except
                return queryset.none()

        return queryset

    def filter_target_types(self, queryset, name, value):  # pylint: disable=unused-argument
        """
        Filter that returns targets using their type.

        **SPECIAL CASE**: course enrollments/generated certificate.

        If the user wants to filter by target_type courseenrollment/generatedcertificate and wants to add filters on
        user or course, it must pass the following:

            - target_type: if the other arguments are passed this is used to differentiate between
            course_id from courseoverview and username from user object.
            Case Course Enrollment
            - enrollment_course_id (optional)
            - enrollment_username (optional)
            Case Generated Certificate
            - certificate_course_id (optional)
            - certificate_username (optional)
        """
        target_id = {}
        username = None
        course_id = None

        if value == "courseenrollment":
            course_id = self.request.query_params.get("enrollment_course_id")
            username = self.request.query_params.get("enrollment_username")

        elif value == "generatedcertificate":
            course_id = self.request.query_params.get("certificate_course_id")
            username = self.request.query_params.get("certificate_username")

        target_id.update({"username": username, "course_id": course_id})

        try:
            if any(object_id for object_id in target_id.values()):
                queryset = queryset.find_all_tags_for(target_type=value, target_id=target_id)
            elif value:
                queryset = queryset.find_all_tags_by_type(value)
        except Exception:  # pylint: disable=broad-except
            return queryset.none()

        return queryset

    def filter_access_type(self, queryset, name, value):  # pylint: disable=unused-argument
        """Filters targets by their access type."""
        if value:
            value_map = {v.lower(): k for k, v in AccessLevel.choices()}
            access = value_map.get(value.lower())
            queryset = queryset.filter(access=access) if access else queryset.none()

        return queryset


class FilterBackend(filters.DjangoFilterBackend):
    """
    Backport this fix (https://github.com/carltongibson/django-filter/pull/1323)
    for range type filters.
    The current version of django-filter on edx-platform (v2.2.0) presents a bug
    were range filters don't produce the correct OpenAPI schema. This schema is
    used by our documentation tools (drf-yasg). This backport should be dropped
    when a new version of django-filter with the fix is released (probably v2.5.0)
    """

    def get_schema_fields(self, view):  # pylint: disable=missing-function-docstring
        # This is not compatible with widgets where the query param differs from the
        # filter's attribute name. Notably, this includes `MultiWidget`, where query
        # params will be of the format `<name>_0`, `<name>_1`, etc...
        assert compat.coreapi is not None, 'coreapi must be installed to use `get_schema_fields()`'  # pylint: disable=no-member
        assert compat.coreschema is not None, 'coreschema must be installed to use `get_schema_fields()`'  # pylint: disable=no-member
        try:
            queryset = view.get_queryset()
        except Exception:  # pylint: disable=broad-except
            queryset = None
            warnings.warn(
                f"{view.__class__} is not compatible with schema generation"
            )

        filterset_class = self.get_filterset_class(view, queryset)
        if not filterset_class:
            return []

        return [self.build_coreapi_field(schema_field_name, field)
                for field_name, field in filterset_class.base_filters.items()
                for schema_field_name in self.get_schema_field_names(field_name, field)
                ]

    def build_coreapi_field(self, name, field):  # pylint: disable=missing-function-docstring
        return compat.coreapi.Field(  # pylint: disable=no-member
            name=name,
            required=field.extra['required'],
            location='query',
            schema=self.get_coreschema_field(field),  # pylint: disable=no-member
        )

    def get_schema_field_names(self, field_name, field):
        """
        Get the corresponding schema field names required to generate the openAPI schema
        by referencing the widget suffixes if available.
        """
        try:
            suffixes = field.field_class.widget.suffixes
        except AttributeError:
            return [field_name]
        else:
            return [field_name] if not suffixes else [
                f'{field_name}_{suffix}'
                for suffix in suffixes if suffix
            ]



""" Backend test abstraction. """


def get_bearer_authentication():
    """Allow to get the function BearerAuthentication from
    https://github.com/eduNEXT/edunext-platform/tree/master/openedx/core/lib/api/authentication.py

    Returns:
        BearerAuthentication function.
    """
    try:
        from openedx.core.lib.api.authentication import BearerAuthentication  # pylint: disable=import-outside-toplevel
    except ImportError:
        BearerAuthentication = object
    return BearerAuthentication



"""
Custom API permissions module
"""
from django.conf import settings
from django.contrib.auth.models import Permission, User
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ImproperlyConfigured
from django.db.utils import ProgrammingError
from rest_framework import exceptions, permissions


def load_permissions():
    """
    Helper method to load a custom permission on DB that will be use to give access
    to eox-tagging API.
    """
    if settings.EOX_TAGGING_LOAD_PERMISSIONS:
        try:
            content_type = ContentType.objects.get_for_model(User)
            Permission.objects.get_or_create(
                codename='can_call_eox_tagging',
                name='Can access eox-tagging API',
                content_type=content_type,
            )
        except (ProgrammingError, ImproperlyConfigured):
            # This code runs when the app is loaded. If a migration has not been done, a
            # ProgrammingError is raised. The ImproperlyConfigured exception typically
            # indicates a configuration issue. We are bypassing these exceptions to allow
            # the migrations to run smoothly when building the Open edX image.
            pass


class EoxTaggingAPIPermission(permissions.BasePermission):
    """
    Defines a custom permissions to access eox-tagging API
    These permissions make sure that a token is created with the client credentials of the same site is being used on.
    """

    def has_permission(self, request, view):
        """
        To grant access, checks if the requesting user either can call eox-tagging API or if it's an admin user.
        """
        if request.user.is_staff:
            return True

        try:
            application_uri_allowed = request.auth.application.redirect_uri_allowed(request.build_absolute_uri('/'))
        except Exception:  # pylint: disable=broad-except
            application_uri_allowed = False

        try:
            client_url_allowed = request.get_host() in request.auth.client.url
        except Exception:  # pylint: disable=broad-except
            client_url_allowed = False

        if client_url_allowed or application_uri_allowed:
            return request.user.has_perm('auth.can_call_eox_tagging')

        # If we get here either someone is using a token created on one site in a different site
        # or there was a missconfiguration of the oauth client.
        # To prevent leaking important information we return the most basic message.
        raise exceptions.NotAuthenticated(detail="Invalid token")



"""
Test Django settings for eox_tagging project.
"""
from __future__ import unicode_literals

import codecs
import os

import yaml

from .common import *  # pylint: disable=wildcard-import, unused-wildcard-import


class SettingsClass:
    """ dummy settings class """


SETTINGS = SettingsClass()
# This is executing the plugin_settings method imported from common module
plugin_settings(SETTINGS)
vars().update(SETTINGS.__dict__)
INSTALLED_APPS = vars().get("INSTALLED_APPS", [])
TEST_INSTALLED_APPS = [
    "django.contrib.sites",
]
for app in TEST_INSTALLED_APPS:
    if app not in INSTALLED_APPS:
        INSTALLED_APPS.append(app)

# For testing
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'db.sqlite3',
    },
}


def plugin_settings(settings):  # pylint: disable=function-redefined
    """
    Set of plugin settings used by the Open Edx platform.
    More info: https://github.com/openedx/edx-platform/blob/master/openedx/core/djangoapps/plugins/README.rst
    """
    settings.EOX_TAGGING_SKIP_VALIDATIONS = True
    settings.EOX_TAGGING_LOAD_PERMISSIONS = False
    settings.EOX_TAGGING_BEARER_AUTHENTICATION = 'eox_tagging.edxapp_wrappers.backends.bearer_authentication_i_v1_test'
    settings.DATA_API_DEF_PAGE_SIZE = 1000
    settings.DATA_API_MAX_PAGE_SIZE = 5000
    settings.EOX_CORE_CERTIFICATES_BACKEND = "eox_core.edxapp_wrapper.backends.certificates_h_v1_test"
    settings.EOX_TAGGING_GET_COURSE_OVERVIEW = "eox_tagging.edxapp_wrappers.backends.course_overview_i_v1_test"
    settings.EOX_TAGGING_GET_ENROLLMENT_OBJECT = "eox_tagging.edxapp_wrappers.backends.enrollment_l_v1_test"
    settings.TEST_SITE = 1

    # setup the databases used in the tutor local environment
    lms_cfg = os.environ.get('LMS_CFG')
    if lms_cfg:
        with codecs.open(lms_cfg, encoding='utf-8') as file:
            env_tokens = yaml.safe_load(file)
        settings.DATABASES = env_tokens['DATABASES']


SETTINGS = SettingsClass()
plugin_settings(SETTINGS)
vars().update(SETTINGS.__dict__)


# Integration test settings
INTEGRATION_TEST_SETTINGS = {
    # Retrieved from the Tutor environment where the integration tests run
    "EOX_TAGGING_BASE_URL": f"http://{os.environ.get('LMS_HOST', 'local.edly.io')}/eox-tagging",
    "API_TIMEOUT": 5,
}



"""
Backend for course enrollments valid for lilac release.
"""
from common.djangoapps.student.models import CourseEnrollment  # pylint: disable=E0401


def get_enrollment_object():
    """Backend to get course enrollment."""
    return CourseEnrollment



"""
Backend CourseOverview file, here are all the methods from
openedx.core.djangoapps.content.course_overviews.
"""
from openedx.core.djangoapps.content.course_overviews.models import CourseOverview  # pylint: disable=import-error


def get_course_overview():
    """Backend to get course overview."""
    return CourseOverview



"""The generic views for the eox-taylor plugin project.

Methods:
    info_view: Show basic plugin information.
"""
from __future__ import unicode_literals

from os.path import dirname, realpath
from subprocess import CalledProcessError, check_output

from django.http import JsonResponse

import eox_tagging


def info_view(request):  # pylint: disable=unused-argument
    """Show basic current plugin information.

    This view allows to identify the current plugin version per
    instance by returning the version number or git commit identifier.

    Returns:
        A dict mapping keys to the corresponding plugin version, name and git commit.
        Example:

        {
            'version': 2.5.4,
            'name': 'eox-tagging',
            'git': 7862374283974923654,
        }
    """
    try:
        working_dir = dirname(realpath(__file__))
        git_data = check_output(['git', 'rev-parse', 'HEAD'], cwd=working_dir)
        git_data = git_data.decode().rstrip('\r\n')
    except CalledProcessError:
        git_data = ''

    return JsonResponse(
        {
            'version': eox_tagging.__version__,
            'name': 'eox-tagging',
            'git': git_data,
        },
    )



"""
Setup file for eox_tagging Django plugin.
"""

from __future__ import print_function

import os
import re

from setuptools import setup

with open("README.rst", "r") as fh:
    README = fh.read()


def load_requirements(*requirements_paths):
    """
    Load all requirements from the specified requirements files.
    Returns a list of requirement strings.
    """
    requirements = set()
    for path in requirements_paths:
        requirements.update(
            line.split('#')[0].strip() for line in open(path).readlines()
            if is_requirement(line)
        )
    return list(requirements)


def is_requirement(line):
    """
    Return True if the requirement line is a package requirement;
    that is, it is not blank, a comment, or editable.
    """
    # Remove whitespace at the start/end of the line
    line = line.strip()

    # Skip blank lines, comments, and editable installs
    return not (
        line == '' or
        line.startswith('-r') or
        line.startswith('#') or
        line.startswith('-e') or
        line.startswith('git+') or
        line.startswith('-c')
    )


def get_version(*file_paths):
    """
    Extract the version string from the file at the given relative path fragments.
    """
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


VERSION = get_version('eox_tagging', '__init__.py')


setup(
    name='eox_tagging',
    version=VERSION,
    description='eox-tagging',
    long_description=README,
    long_description_content_type='text/x-rst',
    license="AGPL",
    author='eduNEXT',
    author_email='contact@edunext.co',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django :: 4.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.11',
    ],
    packages=[
        'eox_tagging',
    ],
    include_package_data=True,
    install_requires=load_requirements('requirements/base.in'),
    zip_safe=False,
    entry_points={
        "lms.djangoapp": [
            'eox_tagging = eox_tagging.apps:EoxTaggingConfig',
        ],
    },
)



"""Module for defining custom fields."""
from rest_framework import serializers


class EnumField(serializers.ChoiceField):
    """Serializer field for choices."""

    def __init__(self, enum, **kwargs):
        self.enum = enum
        kwargs['choices'] = [(e.value, e.name) for e in enum]
        super().__init__(**kwargs)

    def to_representation(self, value):
        """Function that helps with choice serialization."""
        try:
            return getattr(value, 'name', self.enum(value).name)
        except AttributeError:
            return self.enum(value).name

    def to_internal_value(self, data):
        """Function that helps with choice deserialization."""
        try:
            return self.enum[data.upper()]
        except KeyError:
            return None



""" Backend abstraction. """
from importlib import import_module

from django.conf import settings


def get_enrollment_object(*args, **kwargs):
    """ Get enrollment object. """
    backend_function = settings.EOX_TAGGING_GET_ENROLLMENT_OBJECT
    backend = import_module(backend_function)
    return backend.get_enrollment_object(*args, **kwargs)


CourseEnrollment = get_enrollment_object()



""" Test classes for Tags filters. """

from django.test import TestCase
from mock import Mock, patch

from eox_tagging.api.v1.filters import TagFilter
from eox_tagging.models import Tag


class FilterTest(TestCase):
    """Test class for custom filters."""

    def setUp(self):
        """setUp class."""
        self.filterset = TagFilter()
        self.filterset.request = Mock()

    @patch.object(Tag, 'objects')
    def test_filter_by_target_object(self, objects_mock):
        """Used to test filtering tags by target object."""
        objects_mock.find_all_tags_for = Mock()
        name = "username"
        value = "test"

        self.filterset.filter_by_target_object(objects_mock, name, value)

        objects_mock.find_all_tags_for.assert_called_with(
            target_type="user",
            target_id={name: value},
        )

    @patch.object(Tag, 'objects')
    def test_filter_target_certificate_by_uuid(self, objects_mock):
        """Used to test filtering tags by target object."""
        objects_mock.find_all_tags_for = Mock()
        value = "abdcefg12345"
        name = "certificate_verify_uuid"

        self.filterset.filter_by_target_object(objects_mock, name, value)

        objects_mock.find_all_tags_for.assert_called_with(
            target_type="generatedcertificate",
            target_id={"verify_uuid": value},
        )

    @patch.object(Tag, 'objects')
    def test_filter_target_enrollment(self, objects_mock):
        """
        Used to test filtering tags depending on the target type. This also filters courseenrollments
        given that we must specify the target type besides the other filter parameters, in this case
        course_id and username.
        """
        objects_mock.find_all_tags_for = Mock()
        value = "courseenrollment"
        name = "target_type"
        self.filterset.request.query_params = {
            "enrollment_course_id": "course_id",
            "enrollment_username": "username",
        }

        self.filterset.filter_target_types(objects_mock, name, value)

        objects_mock.find_all_tags_for.assert_called_with(
            target_type=value,
            target_id={"username": "username", "course_id": "course_id"},
        )

    @patch.object(Tag, 'objects')
    def test_filter_target_certificate(self, objects_mock):
        """
        Used to test filtering tags depending on the target type. This also filters generatedcertificate
        given that we must specify the target type besides the other filter parameters, in this case
        course_id and username.
        """
        objects_mock.find_all_tags_for = Mock()
        value = "generatedcertificate"
        name = "target_type"
        self.filterset.request.query_params = {
            "certificate_course_id": "course_id",
            "certificate_username": "username",
        }

        self.filterset.filter_target_types(objects_mock, name, value)

        objects_mock.find_all_tags_for.assert_called_with(
            target_type=value,
            target_id={"username": "username", "course_id": "course_id"},
        )

    @patch.object(Tag, 'objects')
    def test_filter_target_enroll_by_user(self, objects_mock):
        """
        Used to test filtering tags depending on the target type. This also filters courseenrollments
        given that we must specify the target type besides the other filter parameters, in this case
        username.
        """
        objects_mock.find_all_tags_for = Mock()
        value = "courseenrollment"
        name = "target_type"
        self.filterset.request.query_params = {
            "enrollment_username": "username",
        }

        self.filterset.filter_target_types(objects_mock, name, value)

        objects_mock.find_all_tags_for.assert_called_with(
            target_type=value,
            target_id={"username": "username", "course_id": None},
        )

    @patch.object(Tag, 'objects')
    def test_filter_target_enroll_by_course(self, objects_mock):
        """
        Used to test filtering tags depending on the target type. This also filters courseenrollments
        given that we must specify the target type besides the other filter parameters, in this case
        course_id.
        """
        objects_mock.find_all_tags_for = Mock()
        value = "courseenrollment"
        name = "target_type"
        self.filterset.request.query_params = {
            "enrollment_course_id": "course_id",
        }

        self.filterset.filter_target_types(objects_mock, name, value)

        objects_mock.find_all_tags_for.assert_called_with(
            target_type=value,
            target_id={"username": None, "course_id": "course_id"},
        )
